@@ -1,5 +1,7 @@
 // Copyright 2018-2025 the Deno authors. MIT license.
 
+use std::sync::OnceLock;
+
 use deno_core::GarbageCollected;
 use deno_core::WebIDL;
 use deno_core::op2;
@@ -43,8 +45,11 @@ pub struct GPUTexture {
   pub error_handler: super::error::ErrorHandler,
 
   pub id: wgpu_core::id::TextureId,
+  // needed by deno
   pub device_id: wgpu_core::id::DeviceId,
+  // needed by deno
   pub queue_id: wgpu_core::id::QueueId,
+  pub default_view_id: OnceLock<wgpu_core::id::TextureViewId>,
 
   pub label: String,
 
@@ -56,8 +61,35 @@ pub struct GPUTexture {
   pub usage: u32,
 }
 
+impl GPUTexture {
+  pub(crate) fn default_view_id(&self) -> wgpu_core::id::TextureViewId {
+    *self.default_view_id.get_or_init(|| {
+      let (id, err) =
+        self
+          .instance
+          .texture_create_view(self.id, &Default::default(), None);
+      if let Some(err) = err {
+        use wgpu_types::error::WebGpuError;
+        assert_ne!(
+          err.webgpu_error_type(),
+          wgpu_types::error::ErrorType::Validation,
+          concat!(
+            "getting default view for a texture ",
+            "caused a validation error (!?)"
+          )
+        );
+        self.error_handler.push_error(Some(err));
+      }
+      id
+    })
+  }
+}
+
 impl Drop for GPUTexture {
   fn drop(&mut self) {
+    if let Some(id) = self.default_view_id.take() {
+      self.instance.texture_view_drop(id).unwrap();
+    }
     self.instance.texture_drop(self.id);
   }
 }
@@ -129,11 +161,9 @@ impl GPUTexture {
     self.usage
   }
   #[fast]
-  fn destroy(&self) -> Result<(), JsErrorBox> {
-    self
-      .instance
-      .texture_destroy(self.id)
-      .map_err(|e| JsErrorBox::generic(e.to_string()))
+  #[undefined]
+  fn destroy(&self) {
+    self.instance.texture_destroy(self.id);
   }
 
   #[cppgc]
@@ -583,7 +613,7 @@ impl From<GPUTextureFormat> for TextureFormat {
         channel: AstcChannel::Unorm,
       },
       GPUTextureFormat::Astc4x4UnormSrgb => Self::Astc {
-        block: AstcBlock::B5x4,
+        block: AstcBlock::B4x4,
         channel: AstcChannel::UnormSrgb,
       },
       GPUTextureFormat::Astc5x4Unorm => Self::Astc {
@@ -693,3 +723,21 @@ impl From<GPUTextureFormat> for TextureFormat {
     }
   }
 }
+
+pub struct GPUExternalTexture {}
+
+impl WebIdlInterfaceConverter for GPUExternalTexture {
+  const NAME: &'static str = "GPUExternalTexture";
+}
+
+// SAFETY: we're sure this can be GCed
+unsafe impl GarbageCollected for GPUExternalTexture {
+  fn trace(&self, _visitor: &mut deno_core::v8::cppgc::Visitor) {}
+
+  fn get_name(&self) -> &'static std::ffi::CStr {
+    c"GPUExternalTexture"
+  }
+}
+
+#[op2]
+impl GPUExternalTexture {}
