@@ -1,7 +1,6 @@
 // Copyright 2018-2025 the Deno authors. MIT license.
 
 pub use deno_cache;
-pub use deno_canvas;
 pub use deno_core;
 pub use deno_cron;
 pub use deno_crypto;
@@ -9,6 +8,7 @@ pub use deno_fetch;
 pub use deno_ffi;
 pub use deno_fs;
 pub use deno_http;
+pub use deno_image;
 pub use deno_io;
 pub use deno_kv;
 pub use deno_napi;
